package sun.nio.ch;

import java.security.AccessController;
import java.security.PrivilegedAction;
import java.nio.channels.SelectableChannel;
import sun.misc.JavaIOFileDescriptorAccess;
import sun.misc.SharedSecrets;
import java.io.IOException;
import java.io.FileDescriptor;

class FileDispatcherImpl extends FileDispatcher
{
    private static final boolean fastFileTransfer;
    private final boolean append;
    
    FileDispatcherImpl(boolean append) {
        this.append = append;
    }
    
    FileDispatcherImpl() {
        this(false);
    }
    
    @Override
    boolean needsPositionLock() {
        return true;
    }
    
    @Override
    int read(FileDescriptor fileDescriptor, long n, int n2) throws IOException {
        return read0(fileDescriptor, n, n2);
    }
    
    @Override
    int pread(FileDescriptor fileDescriptor, long n, int n2, long n3) throws IOException {
        return pread0(fileDescriptor, n, n2, n3);
    }
    
    @Override
    long readv(FileDescriptor fileDescriptor, long n, int n2) throws IOException {
        return readv0(fileDescriptor, n, n2);
    }
    
    @Override
    int write(FileDescriptor fileDescriptor, long n, int n2) throws IOException {
        return write0(fileDescriptor, n, n2, this.append);
    }
    
    @Override
    int pwrite(FileDescriptor fileDescriptor, long n, int n2, long n3) throws IOException {
        return pwrite0(fileDescriptor, n, n2, n3);
    }
    
    @Override
    long writev(FileDescriptor fileDescriptor, long n, int n2) throws IOException {
        return writev0(fileDescriptor, n, n2, this.append);
    }
    
    @Override
    long seek(FileDescriptor fileDescriptor, long n) throws IOException {
        return seek0(fileDescriptor, n);
    }
    
    @Override
    int force(FileDescriptor fileDescriptor, boolean b) throws IOException {
        return force0(fileDescriptor, b);
    }
    
    @Override
    int truncate(FileDescriptor fileDescriptor, long n) throws IOException {
        return truncate0(fileDescriptor, n);
    }
    
    @Override
    long size(FileDescriptor fileDescriptor) throws IOException {
        return size0(fileDescriptor);
    }
    
    @Override
    int lock(FileDescriptor fileDescriptor, boolean b, long n, long n2, boolean b2) throws IOException {
        return lock0(fileDescriptor, b, n, n2, b2);
    }
    
    @Override
    void release(FileDescriptor fileDescriptor, long n, long n2) throws IOException {
        release0(fileDescriptor, n, n2);
    }
    
    @Override
    void close(FileDescriptor fileDescriptor) throws IOException {
        close0(fileDescriptor);
    }
    
    @Override
    FileDescriptor duplicateForMapping(FileDescriptor fileDescriptor) throws IOException {
        JavaIOFileDescriptorAccess javaIOFileDescriptorAccess = SharedSecrets.getJavaIOFileDescriptorAccess();
        FileDescriptor fileDescriptor2 = new FileDescriptor();
        javaIOFileDescriptorAccess.setHandle(fileDescriptor2, duplicateHandle(javaIOFileDescriptorAccess.getHandle(fileDescriptor)));
        return fileDescriptor2;
    }
    
    @Override
    boolean canTransferToDirectly(SelectableChannel selectableChannel) {
        return FileDispatcherImpl.fastFileTransfer && selectableChannel.isBlocking();
    }
    
    @Override
    boolean transferToDirectlyNeedsPositionLock() {
        return true;
    }
    
    static boolean isFastFileTransferRequested() {
        String s = (String)AccessController.doPrivileged(new PrivilegedAction<String>() {
            @Override
            public String run() {
                return System.getProperty("jdk.nio.enableFastFileTransfer");
            }
        });
        return "".equals(s) || Boolean.parseBoolean(s);
    }
    
    static native int read0(FileDescriptor p0, long p1, int p2) throws IOException;
    
    static native int pread0(FileDescriptor p0, long p1, int p2, long p3) throws IOException;
    
    static native long readv0(FileDescriptor p0, long p1, int p2) throws IOException;
    
    static native int write0(FileDescriptor p0, long p1, int p2, boolean p3) throws IOException;
    
    static native int pwrite0(FileDescriptor p0, long p1, int p2, long p3) throws IOException;
    
    static native long writev0(FileDescriptor p0, long p1, int p2, boolean p3) throws IOException;
    
    static native long seek0(FileDescriptor p0, long p1) throws IOException;
    
    static native int force0(FileDescriptor p0, boolean p1) throws IOException;
    
    static native int truncate0(FileDescriptor p0, long p1) throws IOException;
    
    static native long size0(FileDescriptor p0) throws IOException;
    
    static native int lock0(FileDescriptor p0, boolean p1, long p2, long p3, boolean p4) throws IOException;
    
    static native void release0(FileDescriptor p0, long p1, long p2) throws IOException;
    
    static native void close0(FileDescriptor p0) throws IOException;
    
    static native void closeByHandle(long p0) throws IOException;
    
    static native long duplicateHandle(long p0) throws IOException;
    
    static {
        IOUtil.load();
        fastFileTransfer = isFastFileTransferRequested();
    }
}
